package de.hsel.spm.baudas.analysis;

import org.junit.jupiter.api.Test;

import java.io.File;
import java.net.URL;
import java.time.Duration;
import java.util.Arrays;
import java.util.Collections;
import java.util.List;
import java.util.Map;
import java.util.concurrent.atomic.AtomicReference;

import static org.junit.jupiter.api.Assertions.assertEquals;
import static org.junit.jupiter.api.Assertions.assertTimeout;

/**
 * Test class for shopping-cart analysis.
 *
 * @author Johannes Theiner
 * @version 0.1
 */
class ShoppingCartTest {

    @Test
    void test100(){
        URL url = getClass().getClassLoader().getResource("kd100.csv");
        assert url != null;

        ShoppingCart cart = new ShoppingCart(new File(url.getFile()));

        AtomicReference<Map<List<String>, List<String>>> result = new AtomicReference<>();
        assertTimeout(Duration.ofSeconds(4), () -> result.set(cart.getResult()));

        assertEquals(Collections.singletonList("Eisenwaren"), result.get().get(Arrays.asList("Handwerkzeuge", "Schliesstechnik")));
        assertEquals(Collections.singletonList("Elektromaterial"), result.get().get(Collections.singletonList("Leuchten")));
        assertEquals(Collections.singletonList("Eisenwaren"), result.get().get(Arrays.asList("Schliesstechnik", "Renovierung")));
        assertEquals(Collections.singletonList("Eisenwaren"), result.get().get(Arrays.asList("Handwerkzeuge", "Schliesstechnik", "Baustoffe")));
    }

    @Test
    void test1000(){
        URL url = getClass().getClassLoader().getResource("kd1000.csv");
        assert url != null;

        ShoppingCart cart = new ShoppingCart(new File(url.getFile()));

        AtomicReference<Map<List<String>, List<String>>> result = new AtomicReference<>();
        assertTimeout(Duration.ofSeconds(4), () -> result.set(cart.getResult()));

        assertEquals(Collections.singletonList("Eisenwaren"), result.get().get(Arrays.asList("Handwerkzeuge", "Schliesstechnik", "Elektromaterial")));
        assertEquals(Collections.singletonList("Elektromaterial"), result.get().get(Arrays.asList("Renovierung", "Leuchten")));
        assertEquals(Collections.singletonList("Sanitaermaterial"), result.get().get(Arrays.asList("Fliesen", "Armaturen")));
        assertEquals(Collections.singletonList("Eisenwaren"), result.get().get(Arrays.asList("Handwerkzeuge", "Schliesstechnik", "Renovierung")));

    }

    @Test
    void test10000(){
        URL url = getClass().getClassLoader().getResource("kd10000.csv");
        assert url != null;

        ShoppingCart cart = new ShoppingCart(new File(url.getFile()));

        AtomicReference<Map<List<String>, List<String>>> result = new AtomicReference<>();
        assertTimeout(Duration.ofSeconds(5), () -> result.set(cart.getResult()));

        assertEquals(Collections.singletonList("Eisenwaren"), result.get().get(Arrays.asList("Handwerkzeuge", "Schliesstechnik", "Elektromaterial")));
        assertEquals(Collections.singletonList("Sanitaermaterial"), result.get().get(Arrays.asList("Eisenwaren", "Fliesen", "Armaturen")));
        assertEquals(Collections.singletonList("Elektromaterial"), result.get().get(Arrays.asList("Renovierung", "Leuchten")));
        assertEquals(Collections.singletonList("Eisenwaren"), result.get().get(Arrays.asList("Handwerkzeuge", "Schliesstechnik", "Baustoffe")));

    }

}
