package de.hsel.spm.baudas.analysis;

import org.junit.jupiter.api.Test;

import java.io.File;
import java.net.URL;
import java.time.Duration;
import java.util.Map;
import java.util.concurrent.atomic.AtomicReference;

import static org.junit.jupiter.api.Assertions.assertEquals;
import static org.junit.jupiter.api.Assertions.assertTimeout;


/**
 * Week WeekOverviewDiagram Analysis Test.
 *
 * @author Johannes Theiner
 * @version 0.1
 * @since 0.1
 **/
class WeekOverviewTest {

    @Test
    void test100() {
        URL url = getClass().getClassLoader().getResource("kd100.csv");
        assert url != null;

        WeekOverview overview = new WeekOverview(new File(url.getFile()));

        AtomicReference<Map<String, Map.Entry<Double, Integer>>> result = new AtomicReference<>();

        assertTimeout(Duration.ofMillis(1), () -> result.set(overview.getResult()));


        assertEquals(2477, result.get().get("Montag").getKey());
        assertEquals(3359, result.get().get("Dienstag").getKey());
        assertEquals(1436, result.get().get("Mittwoch").getKey());
        assertEquals(2472, result.get().get("Donnerstag").getKey());
        assertEquals(2399, result.get().get("Freitag").getKey());
        assertEquals(18273, result.get().get("Samstag").getKey());

        assertEquals(3216, result.get().get("Montag").getValue());
        assertEquals(5503, result.get().get("Dienstag").getValue());
        assertEquals(1888, result.get().get("Mittwoch").getValue());
        assertEquals(3568, result.get().get("Donnerstag").getValue());
        assertEquals(2953, result.get().get("Freitag").getValue());
        assertEquals(26243, result.get().get("Samstag").getValue());
    }

    @Test
    void test1000() {
        URL url = getClass().getClassLoader().getResource("kd1000.csv");
        assert url != null;

        WeekOverview overview = new WeekOverview(new File(url.getFile()));

        AtomicReference<Map<String, Map.Entry<Double, Integer>>> result = new AtomicReference<>();

        assertTimeout(Duration.ofMillis(3), () -> result.set(overview.getResult()));


        assertEquals(26273, result.get().get("Montag").getKey());
        assertEquals(23345, result.get().get("Dienstag").getKey());
        assertEquals(25203, result.get().get("Mittwoch").getKey());
        assertEquals(26332, result.get().get("Donnerstag").getKey());
        assertEquals(26413, result.get().get("Freitag").getKey());
        assertEquals(146279, result.get().get("Samstag").getKey());

        assertEquals(39839, result.get().get("Montag").getValue());
        assertEquals(34992, result.get().get("Dienstag").getValue());
        assertEquals(37299, result.get().get("Mittwoch").getValue());
        assertEquals(35005, result.get().get("Donnerstag").getValue());
        assertEquals(38911, result.get().get("Freitag").getValue());
        assertEquals(214051, result.get().get("Samstag").getValue());
    }

    @Test
    void test10000() {
        URL url = getClass().getClassLoader().getResource("kd10000.csv");
        assert url != null;

        WeekOverview overview = new WeekOverview(new File(url.getFile()));

        AtomicReference<Map<String, Map.Entry<Double, Integer>>> result = new AtomicReference<>();

        assertTimeout(Duration.ofMillis(30), () -> result.set(overview.getResult()));


        assertEquals(295688, result.get().get("Montag").getKey());
        assertEquals(267269, result.get().get("Dienstag").getKey());
        assertEquals(269610, result.get().get("Mittwoch").getKey());
        assertEquals(254510, result.get().get("Donnerstag").getKey());
        assertEquals(330907, result.get().get("Freitag").getKey());
        assertEquals(1410789, result.get().get("Samstag").getKey());

        assertEquals(426952, result.get().get("Montag").getValue());
        assertEquals(379125, result.get().get("Dienstag").getValue());
        assertEquals(373891, result.get().get("Mittwoch").getValue());
        assertEquals(356996, result.get().get("Donnerstag").getValue());
        assertEquals(480062, result.get().get("Freitag").getValue());
        assertEquals(2015766, result.get().get("Samstag").getValue());
    }
}
